#include <time.h>

#include "base.h"

Board::Board()
	: listener(NULL), level(NULL), grid(NULL), wrapped(false),
	  width(6), height(5)
{
	setupGrid();

	loadLevel(DATA_PREFIX "levels/default.dat", 0);
}

Board::~Board()
{
	clear();
}

void
Board::setupGrid()
{
	grid = new Cell**[height];

	for (size_t i = 0; i < height; i++)
	{
		grid[i] = new Cell*[width];

		memset(grid[i], 0, width);
	}
}

void
Board::clear()
{
	if (grid == NULL)
		return;

	activeComputers = 0;
	totalComputers  = 0;

	for (size_t i = 0; i < getHeight(); i++)
		delete [] grid[i];

	delete [] grid;

	grid = NULL;
}

void
Board::setWidth(size_t width)
{
	this->width = width;
}

void
Board::setHeight(size_t height)
{
	this->height = height;
}

size_t
Board::getWidth() const
{
	return width;
}

size_t
Board::getHeight() const
{
	return height;
}

void
Board::setWrapped(bool wrapped)
{
	this->wrapped = wrapped;
}

bool
Board::isWrapped() const
{
	return wrapped;
}

void
Board::setCell(size_t x, size_t y, Cell *cell)
{
	if (x > width || y > height)
		return;

	grid[y][x] = cell;
}

Cell *
Board::getCell(size_t x, size_t y) const
{
	if (grid == NULL)
		return NULL;

	if (x >= getWidth() || y >= getHeight())
		return NULL;

	if (grid[y] == NULL)
		return NULL;

	return grid[y][x];
}

Cell *
Board::getNearCell(const Cell *baseCell, Direction dir) const
{
	int x2, y2;

	if (baseCell == NULL)
		return NULL;

	x2 = baseCell->getX();
	y2 = baseCell->getY();

	switch (dir)
	{
		case NORTH: y2--; break;
		case EAST:  x2++; break;
		case SOUTH: y2++; break;
		case WEST:  x2--; break;
		default:
			return NULL;
	}

	/* Bounds checking */
	if (x2 == -1)
	{
		if (isWrapped())
			x2 = getWidth() - 1;
		else
			return NULL;
	}
	else if (x2 == (int)getWidth())
	{
		if (isWrapped())
			x2 = 0;
		else
			return NULL;
	}

	if (y2 == -1)
	{
		if (isWrapped())
			y2 = getHeight() - 1;
		else
			return NULL;
	}
	else if (y2 == (int)getHeight())
	{
		if (isWrapped())
			y2 = 0;
		else
			return NULL;
	}

	return getCell(x2, y2);
}

void
Board::setUpdateListener(UpdateListener *listener)
{
	this->listener = listener;
}

UpdateListener *
Board::getUpdateListener() const
{
	return listener;
}

void
Board::setTotalComputers(size_t count)
{
	this->totalComputers = count;
}

size_t
Board::getTotalComputers() const
{
	return totalComputers;
}

void
Board::incNumComputers()
{
	if (activeComputers == getTotalComputers())
		return;

	activeComputers++;
}

void
Board::decNumComputers()
{
	if (activeComputers == 0)
		return;

	activeComputers--;
}

bool
Board::isLevelCompleted() const
{
//	printf("%d -- %d\n", activeComputers, totalComputers);
	return (getTotalComputers() == activeComputers);
}

void
Board::updateCell(Cell *cell)
{
	if (cell == NULL || listener == NULL)
		return;

	listener->cellUpdated(cell->getX(), cell->getY(), cell);
}

void
Board::loadLevel(const char *filename, size_t num)
{
	if (level == NULL)
		delete level;

	clear();

	/* Try the Tickypip Level loader. */
	level = new TPLevel(this);

	if (!level->loadLevelFile(filename))
	{
		delete level;
		level = NULL;

		level = new NetWalkLevel(this);

		if (!level->loadLevelFile(filename))
		{
			delete level;
			level = NULL;

			return;
		}
	}

	level->setupLevel(num);
}

Level *
Board::getLevel() const
{
	return level;
}

void
Board::randomize()
{
	size_t x, y;

	srand(time(NULL));

	for (y = 0; y < getHeight(); y++)
	{
		for (x = 0; x < getWidth(); x++)
		{
			Cell *cell = getCell(x, y);

			if (cell != NULL)
			{
				cell->setDirection(
					(Direction)((double)NUM_DIRECTIONS * rand() /
								(RAND_MAX + 1.0)));
			}
		}
	}
}
