#include "base.h"

Cell::Cell()
	: dir(NORTH), sourceCount(0), board(NULL), x(0), y(0)
{
	for (int i = 0; i < NUM_DIRECTIONS; i++)
		activeSources[i] = false;
}

void
Cell::setX(size_t x)
{
	this->x = x;
}

void
Cell::setY(size_t y)
{
	this->y = y;
}

size_t
Cell::getX() const
{
	return x;
}

size_t
Cell::getY() const
{
	return y;
}

void
Cell::setBoard(Board *board)
{
	this->board = board;
}

Board *
Cell::getBoard() const
{
	return board;
}

void
Cell::rotateRight()
{
	setDirection(DIR_ROTATE_RIGHT(getDirection()));
}

void
Cell::rotateLeft()
{
	setDirection(DIR_ROTATE_LEFT(getDirection()));
}

void
Cell::setDirection(Direction dir)
{
	size_t i;

	bool oldActive = isActive();

	this->dir = dir;

	sourceCount = 0;

	if (getBoard() == NULL)
		return;

	for (i = 0; i < NUM_DIRECTIONS; i++)
	{
		Cell *cell = getBoard()->getNearCell(this, (Direction)i);

		if (cell != NULL && isConnectedWith(cell) && cell->isActive() &&
			!cell->isSource(DIR_OPPOSITE(i)))
		{
			activeSources[i] = true;
			sourceCount++;
        }
		else
			activeSources[i] = false;
    }

	for (i = 0; i < NUM_DIRECTIONS; i++)
	{
		Cell *cell = getBoard()->getNearCell(this, (Direction)i);

		if (cell != NULL && !isSource((Direction)i))
		{
			cell->setSource(DIR_OPPOSITE(i),
							isConnectedWith(cell) && isActive());
		}
	}

	getBoard()->updateCell(this);

	if (oldActive != isActive())
		activeChanged();
}

Direction
Cell::getDirection() const
{
	return dir;
}

bool
Cell::isConnectedWith(const Cell *cell) const
{
	int d;

	if (getBoard() == NULL)
		return false;

	for (d = 0; d < NUM_DIRECTIONS; d++)
	{
		if (cell == getBoard()->getNearCell(this, (Direction)d))
		{
			if (hasConnector((Direction)d) &&
				cell->hasConnector(DIR_OPPOSITE(d)))
			{
				return true;
			}

			return false;
		}
	}

	return false;
}

bool
Cell::isActive() const
{
	if (getBoard() == NULL)
		return false;

	return (sourceCount > 0);
}

void
Cell::paint(QPainter &paint)
{
	Cell *cell;

	paint.fillRect(0, 0, WIDTH, HEIGHT, QColor(230, 230, 230));

	paint.setPen(QColor(0, 0, 0));

	cell = getBoard()->getNearCell(this, SOUTH);

	if (getX() == 0)
	{
		if (cell != NULL && cell->getType() == BLANK)
			paint.drawLine(-1, -1, -1, HEIGHT - 1);
		else
			paint.drawLine(-1, -1, -1, HEIGHT);
	}
	else if (getX() == getBoard()->getWidth() - 1)
	{
		paint.drawLine(WIDTH, -1, WIDTH, HEIGHT - 1);
		paint.setPen(QColor(164, 164, 164));

		if (cell != NULL && cell->getType() == BLANK)
			paint.drawLine(WIDTH + 1, 0, WIDTH + 1, HEIGHT);
		else
			paint.drawLine(WIDTH + 1, 0, WIDTH + 1, HEIGHT + 1);

		paint.setPen(QColor(0, 0, 0));
	}

	cell = getBoard()->getNearCell(this, EAST);

	if (getY() == 0)
	{
		if (cell != NULL && cell->getType() == BLANK)
			paint.drawLine(-1, -1, WIDTH - 1, -1);
		else
			paint.drawLine(-1, -1, WIDTH, -1);
	}
	else if (getY() == getBoard()->getHeight() - 1)
	{
		paint.drawLine(-1, HEIGHT, WIDTH, HEIGHT);
		paint.setPen(QColor(164, 164, 164));

		if (cell != NULL && cell->getType() == BLANK)
			paint.drawLine(0, HEIGHT + 1, WIDTH, HEIGHT + 1);
		else
			paint.drawLine(0, HEIGHT + 1, WIDTH + 1, HEIGHT + 1);
	}

	paint.setPen(QColor(82, 82, 82));

	if (getY() < getBoard()->getHeight() - 1)
	{
		paint.drawLine(0, HEIGHT - 1, WIDTH - 1, HEIGHT - 1);
	}

	if (getX() < getBoard()->getWidth() - 1)
	{
		paint.drawLine(WIDTH - 1, 0, WIDTH - 1, HEIGHT - 1);
	}
}

void
Cell::setSource(Direction dir, bool source)
{
	if (source == activeSources[dir])
		return;

	bool oldActive = isActive();

	if (source)
		sourceCount++;
	else
		sourceCount--;

	activeSources[dir] = source;

	for (int i = 0; i < NUM_DIRECTIONS; i++)
	{
		Cell *cell = getBoard()->getNearCell(this, (Direction)i);

		if (cell != NULL && cell->isConnectedWith(this) &&
			(isActive() ? !isSource((Direction)i)
			            : cell->isSource(DIR_OPPOSITE(i))))
		{
			cell->setSource(DIR_OPPOSITE(i), isActive());
		}
	}

	getBoard()->updateCell(this);

	if (oldActive != isActive())
		activeChanged();
}

bool
Cell::isSource(Direction dir)
{
	if (getBoard() == NULL)
		return false;

	return activeSources[dir];
}
