#include "NetWalkLevel.h"
#include "BlankCell.h"
#include "ComputerCell.h"
#include "ConnectorCell.h"
#include "ServerCell.h"

#include <qtextstream.h>

NetWalkLevel::NetWalkLevel(Board *board)
	: Level(board), topServerCell(NULL)
{
}
NetWalkLevel::NetWalkLevel(QString &name, Board *board)
	: Level(name, board), topServerCell(NULL)
{
}


bool
NetWalkLevel::setupLevel(size_t level)
{
	if (getBoard() == NULL)
		return false;

	unsigned char levelType;
	unsigned long offset;

	topServerCell = NULL;

	/* Find the offset of the record. */
	offset = getRecordOffset();

	f.at(offset + 6 + (level * 92));

	levelType = f.getch();
	f.getch(); /* Skip over 00 */

	getBoard()->clear();

	if ((levelType & 0x0F) == 2)
		getBoard()->setWrapped(true);
	else
		getBoard()->setWrapped(false);

	getBoard()->setWidth(10);
	getBoard()->setHeight(9);
	getBoard()->setupGrid();

	for (size_t y = 0; y < 9; y++)
	{
		for (size_t x = 0; x < 10; x++)
		{
			addCell(f.getch(), x, y);
		}
	}

	topServerCell->setBoard(getBoard());
	topServerCell->setDirection(topServerCell->getDirection());

	return true;
}

unsigned long
NetWalkLevel::getRecordOffset()
{
	char data[4];
	char newData[4];
	unsigned long offset;

	f.at(0x4E);
	f.readBlock(data, 4);

	newData[0] = data[3];
	newData[1] = data[2];
	newData[2] = data[1];
	newData[3] = data[0];

	memcpy(&offset, newData, 4);

	return offset;
}

bool
NetWalkLevel::readHeader()
{
	char numLevels;
	char name[32];
	long offset;
	char *title, *c;

	/* Skip past the title */
	f.at(0);
	f.readBlock(name, 32);

	/*
	 * We're not trusting the title. Name this as the filename without
	 * the extension.
	 */
	title = strdup(getTitle());

	if ((c = strchr(title, '.')) != NULL) {
		*c = '\0';
		setTitle(title);
	}

	delete title;

	/* Find the offset of the record. */
	offset = getRecordOffset();

	/* Get the number of levels */
	f.at(offset + 1);
	numLevels = f.getch();
	setNumLevels(numLevels);

	return true;
}

bool
NetWalkLevel::isValidFormat()
{
	char version[2];
	char dbType[4];
	char creatorId[4];

	f.at(34);

	/* Check the format version. */
	f.readBlock(version, 2);

	if (version[0] != 0 || version[1] != 1)
		return false;

	f.at(f.at() + 24);

	/* Check the Database Type */
	f.readBlock(dbType, 4);

	if (dbType[0] != 'N' || dbType[1] != 'T' ||
		dbType[2] != 'W' || dbType[3] != 'K')
	{
		return false;
	}

	/* Check the creator ID */
	f.readBlock(creatorId, 4);

	if (creatorId[0] != 'B' || creatorId[1] != 'K' ||
		creatorId[2] != 'S' || creatorId[3] != '1')
	{
		return false;
	}
	
	return true;
}

void
NetWalkLevel::addCell(char cellType, size_t x, size_t y)
{
	Cell *cell;
	Direction dir = NORTH;

	switch (cellType)
	{
		/* Blank cell */
		case 0x00: 
			cell = new BlankCell();
			break;

		/* Computer terminals */
		case 0x01: /* N */
		case 0x02: /* E */
		case 0x04: /* S */
		case 0x08: /* W */
			cell = new ComputerCell();
			getBoard()->setTotalComputers(getBoard()->getTotalComputers() + 1);
			
			switch (cellType)
			{
				case 0x01: dir = NORTH; break;
				case 0x02: dir = EAST;  break;
				case 0x04: dir = SOUTH; break;
				case 0x08: dir = WEST;  break;
			}

			cell->setDirection(dir);
			break;

		/* L Connectors */
		case 0x03: /* NE */
		case 0x06: /* SE */
		case 0x09: /* NW */
		case 0x0C: /* SW */
		{
			ConnectorCell *conCell = new ConnectorCell();
			conCell->setConnectorType(ConnectorCell::CONNECTOR_L);
			cell = conCell;
			
			switch (cellType)
			{
				case 0x03: dir = NORTH; break;
				case 0x06: dir = EAST;  break;
				case 0x09: dir = WEST;  break;
				case 0x0C: dir = SOUTH; break;
			}

			cell->setDirection(dir);

			break;
		}

		/* T Connectors */
		case 0x07: /* NES */
		case 0x0B: /* NEW */
		case 0x0D: /* NSW */
		case 0x0E: /* ESW */
		{
			ConnectorCell *conCell = new ConnectorCell();
			conCell->setConnectorType(ConnectorCell::CONNECTOR_T);
			cell = conCell;
			
			switch (cellType)
			{
				case 0x07: dir = EAST;  break;
				case 0x0B: dir = NORTH; break;
				case 0x0D: dir = WEST;  break;
				case 0x0E: dir = SOUTH; break;
			}

			cell->setDirection(dir);
			break;
		}

		/* Line Connectors */
		case 0x05: /* NS */
		case 0x0A: /* EW */
		{
			ConnectorCell *conCell = new ConnectorCell();
			conCell->setConnectorType(ConnectorCell::CONNECTOR_LINE);
			cell = conCell;
			cell->setDirection((cellType == 0x05 ? NORTH : EAST));
			break;
		}

		/* Top server */
		case 0x20:
		case 0x23:
		{
			topServerCell = new ServerCell(ServerCell::SERVER_TOP);
			cell = topServerCell;

			if (cellType == 0x23)
			{
				topServerCell->setConnectorType(ConnectorCell::CONNECTOR_END);
				topServerCell->setDirection(NORTH);
			}
			else
			{
				topServerCell->setConnectorType(ConnectorCell::CONNECTOR_NONE);
			}

			break;
		}

		/* Bottom servers, single directions. */
		case 0x31: /* N */
		case 0x32: /* E */
		case 0x33: /* E */ /* Makes the top 0x33 */
		case 0x34: /* S */
		case 0x38: /* W */
		{
			ServerCell *serverCell = new ServerCell(ServerCell::SERVER_BOTTOM);
			cell = serverCell;

			if (cellType == 0x33 ||
				topServerCell->getConnectorType() ==
				ConnectorCell::CONNECTOR_END)
			{
				topServerCell->setConnectorType(ConnectorCell::CONNECTOR_L);
				serverCell->setConnectorType(ConnectorCell::CONNECTOR_L);

				switch (cellType)
				{
					case 0x31:
					case 0x33: dir = NORTH; break;
					case 0x38: dir = SOUTH; break;
				}
			}
			else
			{
				topServerCell->setConnectorType(ConnectorCell::CONNECTOR_END);
				serverCell->setConnectorType(ConnectorCell::CONNECTOR_END);

				switch (cellType)
				{
					case 0x31: dir = NORTH; break;
					case 0x32: dir = EAST;  break;
					case 0x34: dir = SOUTH; break;
					case 0x38: dir = WEST;  break;
				}
			}

			cell->setDirection(dir);
			topServerCell->setDirection(dir);
			break;
		}

		/* Bottom servers, L connectors */
		case 0x36: /* ES */
		case 0x39: /* NW */
		           /* NE */
		case 0x3C: /* SW */
		{
			ServerCell *serverCell = new ServerCell(ServerCell::SERVER_BOTTOM);
			cell = serverCell;
			serverCell->setConnectorType(ConnectorCell::CONNECTOR_L);
			topServerCell->setConnectorType(ConnectorCell::CONNECTOR_L);

			switch (cellType)
			{
				case 0x36: dir = EAST;  break;
				case 0x39: dir = WEST;  break;
				case 0x3C: dir = SOUTH; break;
			}

			cell->setDirection(dir);
			topServerCell->setDirection(dir);
			break;
		}

		/* Bottom servers, T connectors */
		case 0x37: /* NES */
		case 0x3B: /* NEW */
		case 0x3D: /* NSW */
		case 0x3E: /* ESW */
		{
			ServerCell *serverCell = new ServerCell(ServerCell::SERVER_BOTTOM);
			cell = serverCell;
			serverCell->setConnectorType(ConnectorCell::CONNECTOR_T);
			topServerCell->setConnectorType(ConnectorCell::CONNECTOR_T);

			switch (cellType)
			{
				case 0x37: dir = EAST;  break;
				case 0x3B: dir = NORTH; break;
				case 0x3D: dir = WEST;  break;
				case 0x3E: dir = SOUTH; break;
			}

			cell->setDirection(dir);
			topServerCell->setDirection(dir);
			break;
		}

		/* Bottom servers, line connectors. */
		case 0x35: /* NS */
		case 0x3A: /* EW */
		{
			ServerCell *serverCell = new ServerCell(ServerCell::SERVER_BOTTOM);
			cell = serverCell;
			serverCell->setConnectorType(ConnectorCell::CONNECTOR_LINE);
			topServerCell->setConnectorType(ConnectorCell::CONNECTOR_LINE);

			switch (cellType)
			{
				case 0x35: dir = NORTH; break;
				case 0x3A: dir = EAST;  break;
			}

			cell->setDirection(dir);
			topServerCell->setDirection(dir);
			break;
		}

		/* Bottom servers, all directions. */
		case 0x3F:
		{
			ServerCell *serverCell = new ServerCell(ServerCell::SERVER_BOTTOM);
			cell = serverCell;
			serverCell->setConnectorType(ConnectorCell::CONNECTOR_CROSS);
			topServerCell->setConnectorType(ConnectorCell::CONNECTOR_CROSS);
			break;
		}

		default:
			cell = NULL;
	}

	getBoard()->setCell(x, y, cell);

	if (cell != NULL)
	{
		cell->setX(x);
		cell->setY(y);

		if (topServerCell != cell)
			cell->setBoard(getBoard());
	}
}
